#!/usr/bin/python

import os
import sys
import re
import shlex

#-------------------------------------------------------------------------------
def Extract_Macro_Vars( macro_string ):
    possible_vars = re.findall("\$\{[A-z0-9_]+\}", macro_string)
    return list(set(possible_vars))

def Replace_Macro_Vars( command, macros ):
    """Replace macro variables in the "macro command"."""

    # EXAMPLE: command = '@macro ${VAR1}="value" ${VAR2}="value"'

    # If it's not a macro, return command unmodifyed
    if not command.startswith("@") and not ' ' in command:
        return command
    else:
        # Split command with shlex -> 
        # rows: ['@macro', '${VAR1}=value', '${VAR2}=value']
        rows = shlex.split(command)

        # macro = "@macro"
        macro = rows[0]

        # Get macro definition for the selected macro
        macro_data = [ x for x in macros if x[0] == macro ][0]
        macro_name = macro_data[0]
        macro_command = macro_data[1]

        # Macro has no arguments, return the macro command unmodified
        if len(rows) == 1:
            return macro_command
        # Macro has arguments -> replace them
        else:
            # macro_vars = [ "${VAR1}=value", "${VAR2}="value" ]
            macro_vars = rows[1:]

            # Default return value:
            result = macro_data[1]

            # Build a dict as dict[var] = value:
            macro_vars_dict = {}
            for variable in macro_vars:
                # for each variable found, split by the first = found
                if '=' in variable:
                    rows2 = variable.split("=", 1)
                    the_var_name = rows2[0]
                    the_var_value = rows2[1]
                    macro_vars_dict[the_var_name] = the_var_value

            # Now: macro_vars_dict = { '${VAR1}': 'value', '${VAR2}': 'value' }
            for key in macro_vars_dict:
                if key in result:
                    result = result.replace( key, macro_vars_dict[key] )
                else:
                    # String not found, missing key
                    pass
      
            # We've replaced each VAR with VALUE, so command now is OK
            return result
    

#-------------------------------------------------------------------------------
def main():

    macros = [
     ( "@gnometerminal", "gnome-terminal -t ${USER} -x ssh ${USER}@${SERVER} ${EXTRA}" ),
     ( "@rdesktop", "rdesktop -g 1024x768 ${SERVER}" )
              ]

    test = '@gnometerminal ${USER}="sromero" ${SERVER}="localhost" ${EXTRA}=""'

    print Replace_Macro_Vars( test, macros )


#-------------------------------------------------------------------------------
if __name__ == "__main__":
    main()
